@@ -1,143 +1,4 @@
 
-#' Import a Python module
-#'
-#' Import the specified Python module for calling from R.
-#'
-#' @param module Module name
-#' @param as Alias for module name (affects names of R classes). Note that
-#'  this is an advanced parameter that should generally only be used
-#'  in package development (since it affects the S3 name of the imported
-#'  class and can therefore interfere with S3 method dispatching).
-#' @param path Path to import from
-#' @param convert `TRUE` to automatically convert Python objects to their R
-#'   equivalent. If you pass `FALSE` you can do manual conversion using the
-#'   [py_to_r()] function.
-#' @param delay_load `TRUE` to delay loading the module until it is first used.
-#'   `FALSE` to load the module immediately. If a function is provided then it
-#'   will be called once the module is loaded. If a list containing `on_load()`
-#'   and `on_error(e)` elements is provided then `on_load()` will be called on
-#'   successful load and `on_error(e)` if an error occurs.
-#'
-#' @details The `import_from_path` function imports a Python module from an
-#'   arbitrary filesystem path (the directory of the specified python script is
-#'   automatically added to the `sys.path`).
-#'
-#' @return A Python module
-#'
-#' @examples
-#' \dontrun{
-#' main <- import_main()
-#' sys <- import("sys")
-#' }
-#'
-#' @export
-import <- function(module, as = NULL, convert = TRUE, delay_load = FALSE) {
-
-  # if there is an as argument then register a filter for it
-  if (!is.null(as)) {
-    register_class_filter(function(classes) {
-      sub(paste0("^", module), as, classes)
-    })
-  }
-
-  # resolve delay load
-  delay_load_environment <- NULL
-  delay_load_priority <- 0
-  delay_load_functions <- NULL
-  if (is.function(delay_load)) {
-    delay_load_functions <- list(on_load = delay_load)
-    delay_load <- TRUE
-  } else if (is.list(delay_load)) {
-    delay_load_environment <- delay_load$environment
-    delay_load_functions <- delay_load
-    if (!is.null(delay_load$priority))
-      delay_load_priority <- delay_load$priority
-    delay_load <- TRUE
-  }
-
-  # normal case (load immediately)
-  if (!delay_load || is_python_initialized()) {
-    # ensure that python is initialized (pass top level module as
-    # a hint as to which version of python to choose)
-    ensure_python_initialized(required_module = module)
-
-    # import the module
-    hookName <- paste("reticulate", module, "load", sep = "::")
-    imported <- py_module_import(module, convert = convert)
-
-    # run load hooks
-    hooks <- getHook(hookName)
-    for (hook in hooks)
-      tryCatch(hook(imported), error = warning)
-
-    # remove hooks (we only want to run on first import)
-    setHook(hookName, list(), "replace")
-
-    # return imported module
-    imported
-  }
-
-  # delay load case (wait until first access)
-  else {
-    if (is.null(.globals$delay_load_module) || (delay_load_priority > .globals$delay_load_priority)) {
-      .globals$delay_load_module <- module
-      .globals$delay_load_environment <- delay_load_environment
-      .globals$delay_load_priority <- delay_load_priority
-    }
-    module_proxy <- new.env(parent = emptyenv())
-    module_proxy$module <- module
-    module_proxy$convert <- convert
-    if (!is.null(delay_load_functions)) {
-      module_proxy$get_module <- delay_load_functions$get_module
-      module_proxy$on_load <- delay_load_functions$on_load
-      module_proxy$on_error <- delay_load_functions$on_error
-    }
-    attr(module_proxy, "class") <- c("python.builtin.module",
-                                     "python.builtin.object")
-    module_proxy
-  }
-}
-
-
-#' @rdname import
-#' @export
-import_main <- function(convert = TRUE) {
-  ensure_python_initialized()
-  import("__main__", convert = convert)
-}
-
-#' @rdname import
-#' @export
-import_builtins <- function(convert = TRUE) {
-  ensure_python_initialized()
-  if (is_python3())
-    import("builtins", convert = convert)
-  else
-    import("__builtin__", convert = convert)
-}
-
-
-#' @rdname import
-#' @export
-import_from_path <- function(module, path = ".", convert = TRUE) {
-
-  # normalize path
-  path <- normalizePath(path)
-
-  # add the path to sys.path if it isn't already there
-  sys <- import("sys", convert = FALSE)
-  if (!path %in% py_to_r(sys$path))
-    sys$path$append(path)
-
-  # import
-  import(module, convert = convert)
-}
-
-
-
-
-
-
 #' @export
 print.python.builtin.object <- function(x, ...) {
   str(x, ...)
@@ -150,7 +11,7 @@ str.python.builtin.object <- function(object, ...) {
   if (!py_available() || py_is_null_xptr(object))
     cat("<pointer: 0x0>\n")
   else
-    cat(py_str(object), "\n", sep="")
+    cat(py_str(object), "\n", sep = "")
 }
 
 #' @export
@@ -425,7 +286,7 @@ as.environment.python.builtin.object <- function(x) {
     names <- sort(names, decreasing = FALSE)
 
     # get the types
-    types <- py_suppress_warnings(py_get_attribute_types(x, names))
+    types <- py_suppress_warnings(py_get_attr_types(x, names))
   }
 
 
@@ -870,6 +731,29 @@ py_set_attr <- function(x, name, value) {
   py_set_attr_impl(x, name, value)
 }
 
+#' The Python None object
+#' 
+#' Get a reference to the Python `None` object.
+#' 
+#' @export
+py_none <- function() {
+  ensure_python_initialized()
+  py_none_impl()
+}
+
+#' Delete an attribute of a Python object
+#' 
+#' @param x A Python object.
+#' @param name The attribute name.
+#' 
+#' @export
+py_del_attr <- function(x, name) {
+  ensure_python_initialized()
+  if (py_is_module_proxy(x))
+    py_resolve_module_proxy(x)
+  py_del_attr_impl(x, name)
+}
+
 #' List all attributes of a Python object
 #'
 #'
@@ -886,7 +770,16 @@ py_list_attributes <- function(x) {
   attrs
 }
 
-
+py_get_attr_types <- function(x,
+                              names,
+                              resolve_properties = FALSE)
+{
+  ensure_python_initialized()
+  if (py_is_module_proxy(x))
+    py_resolve_module_proxy(x)
+  
+  py_get_attr_types_impl(x, names, resolve_properties)
+}
 
 #' Get an item from a Python object
 #'
@@ -1174,17 +1067,20 @@ py_flush_output <- function() {
 
 #' Run Python code
 #'
-#' Execute code within the the \code{__main__} Python module.
+#' Execute code within the scope of the \code{__main__} Python module.
 #'
 #' @inheritParams import
-#' @param code Code to execute
-#' @param file Source file
-#' @param local Whether to create objects in a local/private namespace (if
-#'   `FALSE`, objects are created within the main module).
+#' 
+#' @param code The Python code to be executed.
+#' @param file The Python script to be executed.
+#' @param local Boolean; should Python objects be created as part of
+#'   a local / private dictionary? If `FALSE`, objects will be created within
+#'   the scope of the Python main module.
 #'
-#' @return For `py_eval()`, the result of evaluating the expression; For
-#'   `py_run_string()` and `py_run_file()`, the dictionary associated with
-#'   the code execution.
+#' @return A Python dictionary of objects. When `local` is `FALSE`, this
+#'   dictionary captures the state of the Python main module after running
+#'   the provided code. Otherwise, only the variables defined and used are
+#'   captured.
 #'
 #' @name py_run
 #'
@@ -1203,7 +1099,43 @@ py_run_file <- function(file, local = FALSE, convert = TRUE) {
   invisible(py_run_file_impl(file, local, convert))
 }
 
-#' @rdname py_run
+#' Evaluate a Python Expression
+#' 
+#' Evaluate a single Python expression, in a way analogous to the Python
+#' `eval()` built-in function.
+#' 
+#' @param code A single Python expression.
+#' @param convert Boolean; automatically convert Python objects to R?
+#' 
+#' @return The result produced by evaluating `code`, converted to an `R`
+#'   object when `convert` is set to `TRUE`.
+#' 
+#' @section Caveats:
+#' 
+#' `py_eval()` only supports evaluation of 'simple' Python expressions.
+#' Other expressions (e.g. assignments) will fail; e.g.
+#' 
+#' ```
+#' > py_eval("x = 1")
+#' Error in py_eval_impl(code, convert) : 
+#'   SyntaxError: invalid syntax (reticulate_eval, line 1)
+#' ```
+#' 
+#' and this mirrors what one would see in a regular Python interpreter:
+#' 
+#' ```
+#' >>> eval("x = 1")
+#' Traceback (most recent call last):
+#'   File "<stdin>", line 1, in <module>
+#'   File "<string>", line 1
+#' x = 1
+#' ^
+#'   SyntaxError: invalid syntax
+#' ```
+#' 
+#' The [py_run_string()] method can be used if the evaluation of arbitrary
+#' Python code is required.
+#' 
 #' @export
 py_eval <- function(code, convert = TRUE) {
   ensure_python_initialized()
@@ -1293,17 +1225,23 @@ py_resolve_module_proxy <- function(proxy) {
   module <- get("module", envir = proxy)
 
   # load and error handlers
+  before_load <- collect_value("before_load")
   on_load <- collect_value("on_load")
   on_error <- collect_value("on_error")
+  
+  # execute before load handler
+  if (is.function(before_load))
+    before_load()
 
-  # perform the import -- capture error and ammend it with
+  # perform the import -- capture error and amend it with
   # python configuration information if we have it
   result <- tryCatch(import(module), error = clear_error_handler())
   if (inherits(result, "error")) {
     if (!is.null(on_error)) {
 
       # call custom error handler
-      on_error(result)
+      if (is.function(on_error))
+        on_error(result)
 
       # error handler can and should call `stop`, this is just a failsafe
       stop("Error loading Python module ", module, call. = FALSE)
@@ -1324,8 +1262,8 @@ py_resolve_module_proxy <- function(proxy) {
   .globals$delay_load_environment <- NULL
   .globals$delay_load_priority <- 0
 
-  # call on_load if specifed
-  if (!is.null(on_load))
+  # call on_load if provided
+  if (is.function(on_load))
     on_load()
 }
 
@@ -1349,8 +1287,13 @@ py_filter_classes <- function(classes) {
   classes
 }
 
-py_inject_r <- function(envir) {
+py_inject_r <- function() {
 
+  # don't inject 'r' if there's already an 'r' object defined
+  main <- import_main(convert = FALSE)
+  if (py_has_attr(main, "r"))
+    return(FALSE)
+  
   # define our 'R' class
   py_run_string("class R(object): pass")
 
@@ -1358,20 +1301,18 @@ py_inject_r <- function(envir) {
   main <- import_main(convert = FALSE)
   R <- main$R
 
-  # extract active knit environment
-  if (is.null(envir)) {
-    .knitEnv <- yoink("knitr", ".knitEnv")
-    envir <- .knitEnv$knit_global
-  }
-
   # define the getters, setters we'll attach to the Python class
   getter <- function(self, code) {
+    envir <- py_resolve_envir()
     object <- eval(parse(text = as_r_value(code)), envir = envir)
     r_to_py(object, convert = is.function(object))
   }
 
   setter <- function(self, name, value) {
-    envir[[as_r_value(name)]] <<- as_r_value(value)
+    envir <- py_resolve_envir()
+    name  <- as_r_value(name)
+    value <- as_r_value(value)
+    assign(name, value, envir = envir)
   }
 
   py_set_attr(R, "__getattr__", getter)
@@ -1381,9 +1322,40 @@ py_inject_r <- function(envir) {
 
   # now define the R object
   py_run_string("r = R()")
+  
+  # remove the 'R' class object
+  py_del_attr(main, "R")
+  
+  # indicate success
+  TRUE
 
 }
 
+py_resolve_envir <- function() {
+  
+  # if an environment has been set, use it
+  envir <- getOption("reticulate.engine.environment")
+  if (is.environment(envir))
+    return(envir)
+  
+  # if we're running in a knitr document, use the knit env
+  if ("knitr" %in% loadedNamespaces()) {
+    .knitEnv <- yoink("knitr", ".knitEnv")
+    envir <- .knitEnv$knit_global
+    if (is.environment(envir))
+      return(envir)
+  }
+  
+  # if we're running in a testthat test, use the rlang reported envir
+  envir <- getOption("rlang_trace_top_env")
+  if (is.environment(envir))
+    return(envir)
+  
+  # otherwise, default to the global environment
+  envir %||% globalenv()
+  
+}
+
 py_inject_hooks <- function() {
   
   builtins <- import_builtins(convert = TRUE)
@@ -1408,4 +1380,51 @@ py_inject_hooks <- function() {
     builtins[[name]] <- input
   }
   
+  # register module import callback
+  useImportHook <- getOption("reticulate.useImportHook", default = is_python3())
+  if (useImportHook) {
+    loader <- import("rpytools.loader", convert = TRUE)
+    loader$initialize(py_module_onload)
+  }
+  
+}
+
+py_module_onload <- function(module) {
+  
+  # log module loading if requested
+  if (getOption("reticulate.logModuleLoad", default = FALSE)) {
+    writeLines(sprintf("Loaded module '%s'", module))
+  }
+  
+  # retrieve and clear list of hooks
+  hookName <- paste("reticulate", module, "load", sep = "::")
+  hooks <- getHook(hookName)
+  setHook(hookName, NULL, action = "replace")
+  
+  # run hooks
+  for (hook in hooks)
+    tryCatch(hook(), error = warning)
+  
+}
+
+py_module_loaded <- function(module) {
+  sys <- import("sys", convert = TRUE)
+  modules <- sys$modules
+  module %in% names(modules)
+}
+
+py_register_load_hook <- function(module, hook) {
+  
+  # if the module is already loaded, just run the hook
+  if (py_module_loaded(module))
+    return(hook())
+  
+  # otherwise, register the hook to be run on next load
+  name <- paste("reticulate", module, "load", sep = "::")
+  setHook(name, hook)
+  
+}
+
+py_set_interrupt <- function() {
+  py_set_interrupt_impl()
 }
